/**
  *****************************************************************************
  * @title		can.c
  * @platform	STM32F103
  * @author		Dmitry Ovcharenko (https://www.smartmode.info/)
  * @version	V1.1.0
  * @date		05.07.2016
  *
  * @brief		 CAN - , -,   
  *
  *******************************************************************************
  * @attention
  *
  *
  * <h2><center>&copy; COPYRIGHT 2016 SmartMODE</center></h2>
*/


#include "stm32f10x.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_can.h"
#include "misc.h"

#include "can.h"


/**
  * @brief    -, CAN
  * @note          STM32F103.
  * @param  None
  * @retval None
  */
void init_CAN(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* CAN GPIOs configuration */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); 		//   AFIO
	RCC_APB2PeriphClockCmd(CAN1_Periph, ENABLE); 				//   

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);		//   CAN-

	//  CAN RX pin
	GPIO_InitStructure.GPIO_Pin   = CAN1_RX_SOURCE;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IPU;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(CAN1_GPIO_PORT, &GPIO_InitStructure);

	//  CAN TX pin
	GPIO_InitStructure.GPIO_Pin   = CAN1_TX_SOURCE;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(CAN1_GPIO_PORT, &GPIO_InitStructure);

	#ifdef CAN1_ReMap
		GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);			//  Can1  PB8, PB9
	#endif

	//  
	CAN_InitTypeDef CAN_InitStructure;

	CAN_DeInit( CAN1);
	CAN_StructInit(&CAN_InitStructure);

	// CAN cell init
	CAN_InitStructure.CAN_TTCM = DISABLE;
	CAN_InitStructure.CAN_ABOM = DISABLE;
	CAN_InitStructure.CAN_AWUM = DISABLE;
	CAN_InitStructure.CAN_NART = ENABLE;
	CAN_InitStructure.CAN_RFLM = DISABLE;
	CAN_InitStructure.CAN_TXFP = DISABLE;
//	CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;			//    
	CAN_InitStructure.CAN_Mode = CAN_Mode_Silent_LoopBack;	//      
	CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;
	CAN_InitStructure.CAN_BS1 = CAN_BS1_13tq;
	CAN_InitStructure.CAN_BS2 = CAN_BS2_2tq;
	CAN_InitStructure.CAN_Prescaler = CAN1_SPEED_PRESCALE;	//   
	CAN_Init(CAN1, &CAN_InitStructure);

	// CAN filter init
	//          ,    
	CAN_FilterInitTypeDef CAN_FilterInitStructure;

	// ,    
/*	CAN_FilterInitStructure.CAN_FilterNumber = 0;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;					//   
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;					//   
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;				//   
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;				//   
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);*/

	//   
	// ,          (  32- )
	CAN_FilterInitStructure.CAN_FilterNumber = 1;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x07E0<<5;				//   
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;					//   
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x07E0<<5;			//   
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;				//   
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);

	// ,         
	CAN_FilterInitStructure.CAN_FilterNumber = 2;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x00D0<<5;				//   
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0D0<<5;			//   
	CAN_FilterInit(&CAN_FilterInitStructure);

	// ,          (  16- )
/*	CAN_FilterInitStructure.CAN_FilterNumber = 3;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x07E0<<5;				//   1
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x00D0<<5;				//   2
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x07E0<<5;			//   1
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0D0<<5;				//   2
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);*/

	// ,          (  32- )
/*	CAN_FilterInitStructure.CAN_FilterNumber = 4;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdList;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x07E0<<5;				//    1
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;					//    1
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x07E5<<5;			//    2
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;				//    2
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);*/

	// ,          (  16- )
/*	CAN_FilterInitStructure.CAN_FilterNumber = 5;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdList;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_16bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh = 0x07E0<<5;				//  1
	CAN_FilterInitStructure.CAN_FilterIdLow = 0x07E5<<5;				//  2
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0700<<5;			//  3
	CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x00D3<<5;			//  4
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);*/

	//   
	// ,           (  32- )
/*	CAN_FilterInitStructure.CAN_FilterNumber = 6;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh =
			(uint16_t)(0x1fbf9000>>13);									//   
	CAN_FilterInitStructure.CAN_FilterIdLow =
			(uint16_t)(0x1fbf9000<<3) | CAN_IDE_32;						//   
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh =
			(uint16_t)(0x1fbf9000>>13);									//   
	CAN_FilterInitStructure.CAN_FilterMaskIdLow =
			(uint16_t)(0x1fbf9000<<3) | CAN_IDE_32;						//   
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO1;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);

	// ,          (  32- )
	CAN_FilterInitStructure.CAN_FilterNumber = 7;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterIdHigh =
			(uint16_t)(0x1AB03000>>13);									//   
	CAN_FilterInitStructure.CAN_FilterIdLow =
			(uint16_t)(0x1AB03000<<3) | CAN_IDE_32;						//   
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh =
			(uint16_t)(0x1AB03000>>13);									//   
	CAN_FilterInitStructure.CAN_FilterMaskIdLow =
			(uint16_t)(0x1AB03000<<3) | CAN_IDE_32;						//   
	CAN_FilterInit(&CAN_FilterInitStructure);*/

	// ,          (  32- )
	CAN_FilterInitStructure.CAN_FilterNumber = 8;						//  ,   0  13
	CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdList;		//   
	CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;	//  ()
	CAN_FilterInitStructure.CAN_FilterIdHigh =
			(uint16_t)(0x1fbf9000>>13);									//    1
	CAN_FilterInitStructure.CAN_FilterIdLow =
			(uint16_t)(0x1fbf9000<<3) | CAN_IDE_32;						//    1
	CAN_FilterInitStructure.CAN_FilterMaskIdHigh =
			(uint16_t)(0x1AB03000>>13);									//    2
	CAN_FilterInitStructure.CAN_FilterMaskIdLow =
			(uint16_t)(0x1AB03000<<3) | CAN_IDE_32;						//    2
	CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO1;		//   FIFO (    )
	CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;				//  
	CAN_FilterInit(&CAN_FilterInitStructure);



	// NVIC Configuration
	//      FIFO0
	// Enable CAN1 RX0 interrupt IRQ channel
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	// CAN FIFO0 message pending interrupt enable
	CAN_ITConfig(CAN1, CAN_IT_FMP0, ENABLE);

	//      FIFO1
	// Enable CAN1 RX1 interrupt IRQ channel
	NVIC_InitStructure.NVIC_IRQChannel = CAN1_RX1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	// CAN FIFO1 message pending interrupt enable
	CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);


}


/**
  * @brief    CAN    FIFO0
  * @note   None
  * @param  None
  * @retval None
  */

void USB_LP_CAN1_RX0_IRQHandler(void)
{
	CanRxMsg RxMessage;

	//   
	RxMessage.DLC = 	0x00;
	RxMessage.ExtId = 	0x00;
	RxMessage.FMI = 	0x00;
	RxMessage.IDE = 	0x00;
	RxMessage.RTR = 	0x00;
	RxMessage.StdId = 	0x00;
	RxMessage.Data [0] = 0x00;
	RxMessage.Data [1] = 0x00;
	RxMessage.Data [2] = 0x00;
	RxMessage.Data [3] = 0x00;
	RxMessage.Data [4] = 0x00;
	RxMessage.Data [5] = 0x00;
	RxMessage.Data [6] = 0x00;
	RxMessage.Data [7] = 0x00;


	if (CAN_GetITStatus(CAN1, CAN_IT_FMP0) != RESET)		//   
	{
		CAN_Receive(CAN1, CAN_FIFO0, &RxMessage);			//    FIFO0

		//    
		//     ,          FIFO0,  -  FIFO1
		if (RxMessage.IDE == CAN_Id_Standard)
		{
			if (RxMessage.StdId == CAN_CMD_Test_Send)		//    ,       
			{
				CAN_Send_Test(CAN_CMD_Test_Ok);
			}
			if (RxMessage.StdId == CAN_CMD_Test_Ok)			//    
			{
				//    ,   
			}
		}
	}
}



/**
  * @brief    CAN    FIFO1
  * @note   None
  * @param  None
  * @retval None
  */

void CAN1_RX1_IRQHandler(void)
{
	CanRxMsg RxMessage;

	//   
	RxMessage.DLC = 	0x00;
	RxMessage.ExtId = 	0x00;
	RxMessage.FMI = 	0x00;
	RxMessage.IDE = 	0x00;
	RxMessage.RTR = 	0x00;
	RxMessage.StdId = 	0x00;
	RxMessage.Data [0] = 0x00;
	RxMessage.Data [1] = 0x00;
	RxMessage.Data [2] = 0x00;
	RxMessage.Data [3] = 0x00;
	RxMessage.Data [4] = 0x00;
	RxMessage.Data [5] = 0x00;
	RxMessage.Data [6] = 0x00;
	RxMessage.Data [7] = 0x00;


	if (CAN_GetITStatus(CAN1, CAN_IT_FMP1) != RESET)		//    FIFO1
	{
		CAN_Receive(CAN1, CAN_FIFO1, &RxMessage);			//    FIFO1

		//    
		//     ,          FIFO0,  -  FIFO1
		if (RxMessage.IDE == CAN_Id_Extended)
		{
			if (RxMessage.ExtId == CAN_CMD_Test_Ext_2)		//    ,       
			{
				CAN_Send_Test_Ext(CAN_CMD_Test_Ext_5);
			}
			if (RxMessage.ExtId == CAN_CMD_Test_Ext_4)			//    
			{
				CAN_Send_Test(CAN_CMD_Test_Ok);
			}
		}
	}

}

/**
  * @brief       ( )
  * @note      
  * @param  None
  * @retval None
  */
void CAN_Send_Test(uint32_t com)
{
	CanTxMsg TxMessage;
	TxMessage.StdId = com;			//   ( )
	TxMessage.ExtId = 0x00;							//     

	TxMessage.IDE = CAN_Id_Standard;				//  
	TxMessage.RTR = CAN_RTR_DATA;					//  
	TxMessage.DLC = 3;								//    3 -   

	TxMessage.Data[0] = 0x00;						//   1
	TxMessage.Data[1] = 0x01;						//   2
	TxMessage.Data[2] = 0x02;						//   3

	CAN_Transmit(CAN1, &TxMessage);
}


/**
  * @brief       ( )
  * @note      
  * @param  None
  * @retval None
  */
void CAN_Send_Test_Ext(uint32_t com)
{
	CanTxMsg TxMessage;
	TxMessage.StdId = 0x00;							//     
	TxMessage.ExtId = (com);						//   ( )


	TxMessage.IDE = CAN_Id_Extended;				//  
	TxMessage.RTR = CAN_RTR_DATA;					//  
	TxMessage.DLC = 3;								//    3 -   

	TxMessage.Data[0] = 0x00;						//   1
	TxMessage.Data[1] = 0x01;						//   2
	TxMessage.Data[2] = 0x02;						//   3

	CAN_Transmit(CAN1, &TxMessage);
}

